// COpenGLView.cpp : implementation of the COpenGLView class
//

#include "stdafx.h"
#include "OpenGL View Class.h"

#include "OpenGL View ClassDoc.h"
#include "COpenGLView.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



const char* const COpenGLView::_ErrorStrings[]= {
				{"No Error"},					// 0
				{"Unable to get a DC"},			// 1
				{"ChoosePixelFormat failed"},	// 2
				{"SelectPixelFormat failed"},	// 3
				{"wglCreateContext failed"},	// 4
				{"wglMakeCurrent failed"},		// 5
				{"wglDeleteContext failed"},	// 6
				{"SwapBuffers failed"},			// 7

		};

/////////////////////////////////////////////////////////////////////////////
// COpenGLView

IMPLEMENT_DYNCREATE(COpenGLView, CView)

BEGIN_MESSAGE_MAP(COpenGLView, CView)
	//{{AFX_MSG_MAP(COpenGLView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_CHAR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COpenGLView construction/destruction

COpenGLView::COpenGLView() :
	 m_hRC(0), m_pDC(0), m_ErrorString(_ErrorStrings[0])

{
	// TODO: add construction code here

	delta_angle=1.0f;

}

COpenGLView::~COpenGLView()
{
}

BOOL COpenGLView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class

	// An OpenGL window must be created with the following flags and must not
    // include CS_PARENTDC for the class style. 
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
  	
	return CView::PreCreateWindow(cs);
}


/////////////////////////////////////////////////////////////////////////////
// COpenGLView drawing


void COpenGLView::OnDraw(CDC* pDC)
{
	COpenGLViewClassDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here

	::glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	PreRenderScene();


	::glPushMatrix();
	RenderScene();
	::glPopMatrix();

	::glFinish();

	if ( FALSE == ::SwapBuffers( m_pDC->GetSafeHdc() ) )
		{
		SetError(7);
		}
	InvalidateRect(0,FALSE);
	GetParent()->PostMessage(WM_PAINT);

}


/////////////////////////////////////////////////////////////////////////////
// COpenGLView diagnostics

#ifdef _DEBUG
void COpenGLView::AssertValid() const
{
	CView::AssertValid();
}

void COpenGLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

COpenGLViewClassDoc* COpenGLView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(COpenGLViewClassDoc)));
	return (COpenGLViewClassDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// COpenGLView message handlers

int COpenGLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here

	InitializeOpenGL();

	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// GL helper functions

void COpenGLView::SetError( int e )
{
	// if there was no previous error,
	// then save this one
	if ( _ErrorStrings[0] == m_ErrorString ) 
		{
		m_ErrorString = _ErrorStrings[e];
		}
}


BOOL COpenGLView::InitializeOpenGL()
{
	// Can we put this in the constructor?
    m_pDC = new CClientDC(this);

    if ( NULL == m_pDC ) // failure to get DC
		{
		SetError(1);
		return FALSE;
		}

	if (!SetupPixelFormat())
		{
        return FALSE;
		}

    //n = ::GetPixelFormat(m_pDC->GetSafeHdc());
    //::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

  //  CreateRGBPalette();

    if ( 0 == (m_hRC = ::wglCreateContext( m_pDC->GetSafeHdc() ) ) )
		{
		SetError(4);
		return FALSE;
		}

    if ( FALSE == ::wglMakeCurrent( m_pDC->GetSafeHdc(), m_hRC ) )
		{
		SetError(5);
		return FALSE;
		}	

	// specify black as clear color
    ::glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
	// specify the back of the buffer as clear depth
    ::glClearDepth( 1.0f );
	// enable depth testing
    ::glEnable( GL_DEPTH_TEST );

	return TRUE;
}


BOOL COpenGLView::SetupPixelFormat()
{
  static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
        1,                              // version number
        PFD_DRAW_TO_WINDOW |            // support window
          PFD_SUPPORT_OPENGL |          // support OpenGL
          PFD_DOUBLEBUFFER,             // double buffered
        PFD_TYPE_RGBA,                  // RGBA type
        24,                             // 24-bit color depth
        0, 0, 0, 0, 0, 0,               // color bits ignored
        0,                              // no alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
//        32,                             // 32-bit z-buffer
		16,	// NOTE: better performance with 16-bit z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0                         // layer masks ignored
    };
    int pixelformat;

    if ( 0 == (pixelformat = ::ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) )
	    {
		SetError(2);
        return FALSE;
		}

    if ( FALSE == ::SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd) )
	    {
       	SetError(3);
        return FALSE;
		}

    return TRUE;
}

void COpenGLView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here

    if ( FALSE == ::wglDeleteContext( m_hRC ) )
		{
		SetError(6);
 		}

    if ( m_pDC )
		{
        delete m_pDC;
		}
}


BOOL COpenGLView::OnEraseBkgnd( CDC* pDC ) 
{
	// TODO: Add your message handler code here and/or call default
	
	//	return CView::OnEraseBkgnd(pDC);
	return TRUE; // tell Windows not to erase the background
}

void COpenGLView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	GLdouble aspect_ratio; // width/height ratio
	
	if ( 0 >= cx || 0 >= cy )
		{
		return;
		}


	SetupViewport( cx, cy );

	// select the projection matrix and clear it
    ::glMatrixMode( GL_PROJECTION );
    ::glLoadIdentity();

	// compute the aspect ratio
	// this will keep all dimension scales equal
	aspect_ratio = (GLdouble)cx/(GLdouble)cy;

	// select the viewing volumn
	SetupViewingFrustum( aspect_ratio );
	
	// switch back to the modelview matrix
    ::glMatrixMode( GL_MODELVIEW );
    ::glLoadIdentity();

	// now perform any viewing transformations
	SetupViewingTransform();
}

 /////////////////////////////////////////////////////////////////////////////
// COpenGLView helper functions

BOOL COpenGLView::SetupViewport( int cx, int cy )
{
	// select the full client area
    ::glViewport(0, 0, cx, cy);

	return TRUE;
}

BOOL COpenGLView::SetupViewingFrustum( GLdouble aspect_ratio )
{
	// select a default viewing volumn
    ::gluPerspective( 40.0f, aspect_ratio, .1f, 20.0f );
	return TRUE;
}


BOOL COpenGLView::SetupViewingTransform()
{
	// select a default viewing transformation
	// of a 20 degree rotation about the X axis
	// then a -5 unit transformation along Z
	::glTranslatef( 0.0f, 0.0f, -5.0f );
	::glRotatef( 20.0f, 1.0f, 0.0f, 0.0f );
    return TRUE;
}


BOOL COpenGLView::RenderScene()
{

	float u=0.0f, v=0.0f,delta_v=0.1f,delta_u=0.1f;
	static float angle=0.0f;

	
	angle+=delta_angle;
	::glRotatef(angle,1.0,0.0,0.0);
	::glRotatef(angle/4.0f,0.0,0.0,1.0);
	::glScalef(2.5,2.5,2.5);
	::glTranslatef(2.25,0.0,-2.0);


	GLfloat bez_pts[]={	-1.6f,0.0f,1.875f,
						-1.6f,0.3f,1.875f,
						-1.5f,0.3f,2.1f,
						-1.5f,0.0f,2.1f,

						-2.3f,0.0f,1.875f,
						-2.3f,0.3f,1.875f,
						-2.5f,0.3f,2.1f,
						-2.5f,0.0f,2.1f,

						-2.7f,0.0f,1.875f,
						-2.7f,0.3f,1.875f,
						-3.0f,0.3f,2.1f,
						-3.0f,0.0f,2.1f,

						-2.7f,0.0f,1.65f,
						-2.7f,0.3f,1.65f,
						-3.0f,0.3f,1.65f,
						-3.0f,0.0f,1.65f};

	::glLineWidth(2);
	::glMap2f(GL_MAP2_VERTEX_3,0.0,1.0,3,4,0.0,1.0,12,4,bez_pts);
	::glEnable(GL_MAP2_VERTEX_3);
	::glColor3f( 1.0f, 1.0f, 1.0f ); 

//Long way to draw mesh
/*	::glBegin(GL_LINES); // each pair of glVertex will form a line
	for(u=0;u<1;u+=delta_u){
		for(v=0;v<1;v+=delta_v){
			glEvalCoord2f(v,u);
			glEvalCoord2f(v,u+delta_u);
			
			glEvalCoord2f(v,u);
			glEvalCoord2f(v+delta_v,u);

		}
	}
	::glEnd();
*/

//easy way to draw mesh	
	glMapGrid2f(10,0.0,1.0,10,0.0,1.0); 
	glEvalMesh2(GL_LINE,0,10,0,10);	

//to draw reverse of mesh, make all y-coord negative
	int i;
	for(i=0;i<48;i+=3)
		bez_pts[i+1]=-bez_pts[i+1];

	::glMap2f(GL_MAP2_VERTEX_3,0.0,1.0,3,4,0.0,1.0,12,4,bez_pts);
	::glEnable(GL_MAP2_VERTEX_3);
	::glColor3f( 1.0f, 0.8f, 0.6f ); 
	::glMapGrid2f(20,0.0,1.0,20,0.0,1.0); 
	::glEvalMesh2(GL_LINE,0,20,0,20);	
	

// draw the control polys
	::glLineWidth(4);

	::glColor3f( 1.0f, 1.0f, 0.0f ); 
	::glBegin(GL_LINE_STRIP);
	for(i=0;i<12;i+=3)
		glVertex3fv(bez_pts+i);// draw the control polys
	::glEnd();

	::glColor3f( 0.0f, 1.0f, 1.0f ); 
	::glBegin(GL_LINE_STRIP);
	for(;i<24;i+=3)
		glVertex3fv(bez_pts+i);// draw the control polys
	::glEnd();

	::glColor3f( 1.0f, 0.0f, 1.0f ); 
	::glBegin(GL_LINE_STRIP);
	for(;i<36;i+=3)
		glVertex3fv(bez_pts+i);// draw the control polys
	::glEnd();

	::glColor3f( 0.66f, 1.0f, 0.66f ); 
	::glBegin(GL_LINE_STRIP);
	for(;i<48;i+=3)
		glVertex3fv(bez_pts+i);// draw the control polys
	::glEnd();

	

    return TRUE;
}	



void COpenGLView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default

	if(nChar==',')
			delta_angle=delta_angle*0.5f;

	if(nChar==' ')
			delta_angle=0.0;

	if(nChar=='.'){
		if(delta_angle<0.1f)
			delta_angle=0.1f;
		delta_angle=delta_angle*1.5f;
	}

	if(nChar=='g')
			delta_angle=1.0;

	if(delta_angle  < 0.0999)
		delta_angle=0.0f;
	CView::OnChar(nChar, nRepCnt, nFlags);
}
